@@ -4,7 +4,7 @@
 /*								*/
 /* Some Define and Structs					*/
 /****************************************************************/
-/* bgp.h,v 1.44 2002/01/31 05:05:46 kuniaki Exp */
+/* bgp.h,v 1.47 2002/04/01 10:01:34 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2002 Internet Initiative Japan Inc.
@@ -47,7 +47,7 @@
 #include <unistd.h>
 #include "utils.h"
 
-#define BV_VERSION	"0.31Alpha"
+#define BV_VERSION	"0.32Alpha"
 #define BV_COPYRIGHT	"Copyright (c) 1998-2002 Internet Initiative Japan Inc."
 
 /* BGP Header Type field defines */
@@ -112,6 +112,14 @@
 #define BGPOPT_CAPABILITY		2	/* RFC2842 */
 
 /* Capability Code -- RFC2842 */
+#define CAP_RESERVE			0	/* RFC2842 */
+#define CAP_MPBGP			1	/* RFC2858 */
+#define CAP_ROUTEREFRESH		2	/* RFC2918 */
+#define CAP_CRFC			3	/* Rekter  */
+#define CAP_MULTIROUTE			4	/* RFC3107 */
+#define CAP_GRACEFUL			64	/* Chen    */
+#define CAP_4OCTAS			65	/* Chen    */
+#define CAP_DCAP			66	/* Chen    */
 
 /* Err Codes */
 #define MESSAGE_HEADER_ERROR		1
@@ -146,7 +154,7 @@
 #define MALFORMED_AS_PATH		11
 
 /* PATH ATTRIBUTE Type Code */
-#define ATTR_MAX			10
+#define ATTR_MAX			18
 #define ATTR_UNKNOWN			0
 #define ATTR_ORIGIN			1
 #define ATTR_AS_PATH			2
@@ -158,6 +166,24 @@
 #define ATTR_COMMUNITIES		8
 #define ATTR_ORIGINATOR_ID		9
 #define ATTR_CLUSTER_LIST		10
+#define ATTR_DPA			11	/* Chen */
+#define ATTR_ADVERTISER			12	/* RFC1863 */
+#define ATTR_RCIDPATH_CLUSTERID		13	/* RFC1863 */
+#define ATTR_MP_REACH_NLRI		14	/* RFC2283 */
+#define ATTR_MP_UNREACH_NLRI		15	/* RFC2283 */
+#define ATTR_EXTENDED_COMMUNITIES	16	/* Rosen */
+#define ATTR_NEW_AS_PATH		17	/* E.Chen */
+#define ATTR_NEW_AGGREGATOR		18	/* E.Chen */
+
+/* AFI / IANA Address Family Numbers */
+#define AF_IPV4			1
+#define AF_IPV6			2
+
+/* SAFI / IANA Subsequence Address Family Identifier RFC2858 */
+#define SAFI_NLRI_UNI		1
+#define SAFI_NLRI_MULTI		2
+#define SAFI_NLRI_BOTH		3
+#define SAFI_NLRI_MPLS		4
 
 /* Default BGP port */
 #define BGP_PORT		179
@@ -180,6 +206,11 @@
 #define PROC_REBOOT		2
 #define PROC_CHILDEND		3
 
+/* BGPView Peer NLRI Types */
+#define NLRI_DEFAULT		0
+#define IPv4_UNICAST		1
+#define IPv6_UNICAST		2
+
 #define DEFSYSNAME		"BGPView"
 
 typedef u_long	net_ulong;	/* network-byte-ordered u_long */
@@ -201,6 +232,7 @@ typedef struct {
 	net_ulong	local_id;		/* local IDENT */
 	u_short		local_as;		/* local AS */
 	char		admin_addr[FNAMESIZE];  /* Peer Admin Email */
+	char		nlri;			/* NLRI Type */
 	int		soc;			/* Socket Discripter */
 	int		flag;			/* Socket Flags	*/
 	int		status;			/* BGP Status (FSM) */
