@@ -4,7 +4,7 @@
 /*                                                              */
 /* Main Routine                                                 */
 /****************************************************************/
-/* bgpview.c,v 1.43 2002/01/31 05:05:46 kuniaki Exp */
+/* bgpview.c,v 1.46 2002/04/01 10:01:34 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2002 Internet Initiative Japan Inc.
@@ -397,7 +397,8 @@ int pn;
 				"Bad BGP Identifier",
 				"Unsupported Optional Parameter",
 				"Authentication Failure",
-				"Unacceptable Hold Time" };
+				"Unacceptable Hold Time",
+				"Unsupported Capability" /* RFC2842 */ };
 
 	static char *ume[] = {	"",
 				"Malformed Attribute List",
@@ -433,7 +434,7 @@ int pn;
 	}
 	sprintf(logstr, "  Error Subcode = %d (%s)\n", (int)idx, submsg);
 	pr_log(VLOG_NOTI, pn, logstr, OFF);
-	pr_log(VLOG_NOTI, pn, "  Datas:\n", OFF);
+	pr_log(VLOG_NOTI, pn, "  Data:\n", OFF);
 	for(ptr = 2;ptr < peer[pn].datalen; ptr++){
 		sprintf(logstr, "%02X",*(peer[pn].data + ptr));
 		pr_log(VLOG_NOTI, pn, logstr, OFF);
@@ -493,11 +494,11 @@ int len;
 	return len - left;
 }
 
-int sendopen(pn)
-int pn;
+int sendopen(int pn)
 {
-	char	buf[256];
+	char	buf[1024];
 	int	sz;
+	int	bt;
 #ifdef DEBUG
 	struct in_addr adr;
 #endif
@@ -510,7 +511,7 @@ int pn;
 	}
 
 	memset(buf, 0xFF, 29);
-	store_int16(buf+16, 29);
+	/* store_int16(buf+16, 29); */
 	*(buf+18) = BGP_OPEN;
 	*(buf+19) = 4;
 	store_int16(buf+20, peer[pn].local_as);
@@ -518,6 +519,23 @@ int pn;
 	store_int32(buf+24, ntohl(peer[pn].local_id));
 	*(buf+28) = '\0';
 
+	/*** Insert MPBGP Option for IPv6 ***/
+	bt = 0;
+	if (peer[pn].nlri == IPv6_UNICAST) {
+		*(buf+bt+29) = BGPOPT_CAPABILITY;
+		*(buf+bt+30) = 6;
+		*(buf+bt+31) = CAP_MPBGP;
+		*(buf+bt+32) = 4;
+		store_int16(buf+33, AF_IPV6);
+		*(buf+bt+35) = 0;
+		*(buf+bt+36) = SAFI_NLRI_UNI;
+		bt += 8;
+	}
+
+	*(buf+28) = bt;
+	store_int16(buf+16, 29+bt);
+
+
 #ifdef DEBUG
 	printf("Sending Open Message :\n");
 	printf("  Socket Number       : %d\n", peer[pn].soc);
@@ -529,9 +547,19 @@ int pn;
 	(void)memcpy((char *)&adr.s_addr, buf+24, 4);
 	printf("  BGP Identifier      : %s\n", inet_ntoa(adr));
 	printf("  Option Length       : %d\n", *(buf+28));
+	if (*(buf+28) != 0) {
+		int optcnt;
+		optcnt = 0;
+		printf("  Option DUMP         : ");
+		while(optcnt < *(buf+28)) {
+			printf("%02X", (u_char)*(buf+29+optcnt));
+			optcnt++;
+		}
+		printf("\n");
+	}
 #endif
 
-	if ((sz = sockwrite(peer[pn].soc, buf, 29)) > 0) {
+	if ((sz = sockwrite(peer[pn].soc, buf, 29+bt)) > 0) {
 		sprintf(logstr,
 			"Open Message Send Succeed: wrote %d byte\n", sz);
 		pr_log(VLOG_OPEN, pn, logstr, ON);
