@@ -4,7 +4,7 @@
 /*                                                              */
 /* FSM Routine                                                  */
 /****************************************************************/
-/* fsm.c,v 1.9 2001/09/26 07:21:23 kuniaki Exp */
+/* fsm.c,v 1.12 2002/03/28 09:54:36 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -53,45 +53,197 @@ extern int sendnotification(/*pn, ec, sc*/);	/* in bgpview.c */
 extern int peer_off_timer(/*id1*/);	/* in timer.c */
 extern void pr_update(/*pn*/);		/* in update.c */
 
+char capability_check(char *param, char length, int pn)
+{
+	u_char	paramcode;
+	char	paramlen;
+	char	paramcnt;
+	char	opterr;
+	char	*ptr;
+	char	pline[LOGSTRSIZE];
+
+
+#ifdef DEBUG
+	char *ptr2,*ptr3;
+	int  cnt_t;
+	/* Capability DUMP */
+	cnt_t = 0;
+	printf("DEBUG: CAPDUMP: LEN: %d Octets\n", length);
+	printf("DEBUG: CAPDUMP 0000: ");
+	ptr2 = param;
+	ptr3 = param+length;
+	while(ptr2 < ptr3) {
+		printf("%08X ", buf2ulong(ptr2));
+		ptr2 += 4;
+		cnt_t++;
+		if ((cnt_t % 4) == 0) {
+			printf("\nDEBUG: CAPDUMP %04d: ", cnt_t*4);
+		}
+	}
+	printf("\n");
+#endif
+	
+
+	paramcnt = 0;
+	opterr = 0;
+	ptr = param;
+	
+	while(paramcnt < length) {
+		paramcode = *ptr;
+		paramlen  = *(ptr+1);
+
+		sprintf(pline, "CAPACHK: Capability Code : %d\n", paramcode);
+		pr_log(VLOG_GLOB, pn, pline);
+
+		switch(paramcode) {
+		case CAP_RESERVE:
+			/* RESERVE */
+			pr_log(VLOG_GLOB, pn, "CAPACHK: Unsupported Capability\n", ON);
+			opterr = UNSUPPORTED_CAPABILITY;
+			break;
+
+		case CAP_MPBGP:
+			/* MPBGP */
+#ifdef DEBUG
+			printf("DEBUG: CAP_MPBGP: AFIDUMP: %02X%02X %02X %02X\n",
+				*(ptr+2), *(ptr+3), *(ptr+4), *(ptr+5));
+#endif
+			sprintf(pline, "CAPACHK: CAP_MPBGP: AFI : %d\n", ntohs(*(u_short *)(ptr+2)));
+			pr_log(VLOG_GLOB, pn, pline);
+
+			switch(ntohs(*(u_short *)(ptr+2))) {
+
+#ifdef HAVE_IPv6
+			case AF_IPV6:
+#ifdef DEBUG
+				printf("DEBUG: CAPACHK: Address Family IPv6\n");
+#endif
+				opterr = 0;
+				break;
+#endif
+			default:
+				pr_log(VLOG_GLOB, pn, "CAPACHK: AFI_CHK: Unsupported Capability\n", ON);
+				opterr = UNSUPPORTED_CAPABILITY;
+			}
+			if (opterr != 0) break;
+
+			sprintf(pline, "CAPACHK: CAP_MPBGP: SAFI : %d\n", (u_char)*(ptr+5));
+			pr_log(VLOG_GLOB, pn, pline);
+
+			switch((u_char)*(ptr+5)) {
+			case SAFI_NLRI_UNI:
+				opterr = 0;
+				break;
+			default:
+				pr_log(VLOG_GLOB, pn, "CAPACHK: Unsupported Capability\n", ON);
+				opterr = UNSUPPORTED_CAPABILITY;
+			}
+			break;
+
+		case CAP_ROUTEREFRESH: /* RFC2918 */
+			/* This Capability is Ignored */
+			break;
+
+		case 128: /* Capability Private Use 128 */
+			pr_log(VLOG_GLOB, pn,
+			  "CAPACHK: Private Capability Code(Cisco?) is Ignored\n", ON);
+			break;
+
+		default:
+			pr_log(VLOG_GLOB, pn, "CAPACHK: Unsupported Capability\n", ON);
+			opterr = UNSUPPORTED_CAPABILITY;
+		}
+		if (opterr != 0) break;
+		paramcnt += (2+paramlen);
+		ptr += (2+paramlen);
+
+#ifdef DEBUG
+		printf("DEBUG: Parameter Length Check = %d / %d\n",
+								 paramcnt, length);
+#endif
+	}
+
+	return(opterr);
+
+}
+
+
 int openerrchk(pn)
 int pn;
 {
 	u_short	bv;
 	char	*ptr;
+	char	optlen;
+
+	char	paramcnt;
+	char	paramcode;
+	char	paramlen;
+	char	*paramval;
+	char	opterr;
 
 	ptr = peer[pn].data;
 	/* Version Check */
 	if ((char)*ptr != 4) {
 		/* Versio  Error */
 		/* Support Version is only 4. */
+		pr_log(VLOG_GLOB, pn, "OPENCHK: Unsupported Version Number\n",ON);
 		return UNSUPPORTED_VERSION_NUMBER;
 	}
+	ptr++;
 
 	/* Autonomous Number Check */
-	if (buf2ushort(ptr+1) != peer[pn].remote_as) {
+	if (buf2ushort(ptr) != peer[pn].remote_as) {
+		pr_log(VLOG_GLOB, pn, "OPENCHK: Bad AS Number\n",ON);
 		return BAD_PEER_AS;
 	}
+	ptr+=2;
 
 	/* Hold Time Error Check */
-	bv = buf2ushort(ptr+3);
+	bv = buf2ushort(ptr);
 	if ((bv == 1) || (bv == 2)) {
 		/* Hold Time Error */
+		pr_log(VLOG_GLOB, pn, "OPENCHK: Unacceptable Hold Time\n",ON);
 		return UNACCEPTABLE_HOLD_TIME;
 	}
+	ptr+=2;
 
 	/* BGP Identifier Error Check */
+	ptr+=4;
 	/* NO CHECK */
 
 	/* Option Parameter Error Check */
-	/* NO CHECK */
+	optlen = *ptr;
+	ptr++;
 
 	/* OPTION PARAMETER CHECK */
-	/* NO CHECK*/
+	paramcnt = 0;
+	opterr = 0;
+	while(paramcnt < optlen) {
+		paramcode = *ptr;
+		paramlen  = *(ptr+1);
 
-	/* Authentication Check */
-	/* NO CHECK */
+		switch(paramcode) {
+		case BGPOPT_AUTHENTICATION:
+			/* Authentication NOT SUPPORT */
+			pr_log(VLOG_GLOB, pn, "OPENCHK: Authentication Failure\n",ON);
+			opterr = AUTHENTICATION_FAILURE;
+			break;
 
-	return 0;
+		case BGPOPT_CAPABILITY:
+			/* Capability Check */
+			opterr = capability_check((ptr+2), paramlen, pn);
+			break;
+
+		default:
+			pr_log(VLOG_GLOB, pn, "OPENCHK: Unsupported Optional Parameter\n",ON);
+			opterr = UNSUPPORTED_OPTIONAL_PARAMETER;
+		}
+		if (opterr != 0) break;
+		paramcnt += (2+paramlen);
+		ptr += (2+paramlen);
+	}
+
+	return(opterr);
 }
 
 void clear_peer(pn)
