@@ -4,7 +4,7 @@
 /*                                                              */
 /* Routeing Information Update Routine                          */
 /****************************************************************/
-/* update.c,v 1.20 2001/09/19 10:27:54 kuniaki Exp */
+/* update.c,v 1.23 2002/04/01 10:01:35 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -38,6 +38,7 @@
 #include "route.h"
 #include "routertt.h"
 #include "timer.h"
+#include "mpnlri.h"
 
 #ifdef RADIX
 #include "radix.h"
@@ -171,6 +172,7 @@ void pr_update(int pn)
 	prefix_t	outpre;
 	char		pathbuf[4096];
 	int		pathbuflen;
+	int		ipv6updcnt, ipv6wdrcnt;
 
 	static char	*path_attr[] = {
 			/* 0 */ "UNKNOWN",
@@ -183,7 +185,15 @@ void pr_update(int pn)
 			/* 7 */ "AGGREGATOR",
 			/* 8 */ "COMMUNITIES",
 			/* 9 */ "ORIGINATOR_ID",
-			/* 10 */ "CLUSTER_LIST" };
+			/* 10 */ "CLUSTER_LIST",
+			/* 11 */ "DPA",
+			/* 12 */ "ADVERTISER",
+			/* 13 */ "RCIDPATH_CLUSTERID",
+			/* 14 */ "MP_REACH_NLRI",
+			/* 15 */ "MP_UNREACH_NLRI",
+			/* 16 */ "EXTENDED_COMMUNITIES",
+			/* 17 */ "NEW_AS_PATH",
+			/* 18 */ "NEW_AGGREGATOR" };
 
 	ptr = peer[pn].data;
 
@@ -555,6 +565,35 @@ void pr_update(int pn)
 				pr_log(VLOG_UPAT, pn, "\n", OFF);
 				ptr += attr_octs;
 				break;
+
+			case ATTR_DPA: /* Chen */
+			case ATTR_ADVERTISER: /* RFC1863 */
+			case ATTR_RCIDPATH_CLUSTERID: /* RFC1863 */
+				/* Unsupport */
+				ptr += attr_octs;
+				break;
+
+			case ATTR_MP_REACH_NLRI: /* RFC2283 */
+				ipv6updcnt = pr_mp_reach_nlri(ptr, attr_octs, pn);
+				sprintf(logstr, "    Received IPv6 Update Routes = %d\n", ipv6updcnt);
+				pr_log(VLOG_UPAT, pn, logstr, OFF);
+				ptr += attr_octs;
+				break;
+
+			case ATTR_MP_UNREACH_NLRI: /* RFC2283 */
+				ipv6wdrcnt = pr_mp_unreach_nlri(ptr, attr_octs, pn);
+				sprintf(logstr, "    Received IPv6 Withdrawn Routes = %d\n", ipv6wdrcnt);
+				pr_log(VLOG_UPAT, pn, logstr, OFF);
+				ptr += attr_octs;
+				break;
+
+			case ATTR_EXTENDED_COMMUNITIES: /* Rosen */
+			case ATTR_NEW_AS_PATH: /* E.Chen */
+			case ATTR_NEW_AGGREGATOR: /* E.Chen */
+
+			default:
+				/* Unknown Attribute Error */
+				ptr += attr_octs;
 			}
 		}
 	}
