@@ -4,7 +4,7 @@
 /*                                                              */
 /* Utility Sub-routines                                         */
 /****************************************************************/
-/* utils.h,v 1.4 2001/10/01 11:12:33 kuniaki Exp */
+/* utils.h,v 1.5 2002/03/28 09:54:37 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -69,5 +69,7 @@ extern void 	store_int32(/*ptr, var*/);
 extern boolean	chkfmt(/* int type, char *str */);
 extern int	checkpw(/* char *phrase, char *pass */);
 extern void 	wait_sigchld();
+extern int 	bintostrv6(/* char *ptr, char *str */);
+extern char 	*strtobinv6(/* char *ptr, char *str */);
 
 #endif /* _UTILS_H */
